{ Test/demo ExtractWord
  ref: DataProblems Maybe
       https://forum.lazarus.freepascal.org/index.php/topic,45150.0.html

  This is example code intended only to test/demonstrate some feature of
  Lazarus and/or Free Pascal. While care has been taken to make it fully
  functional, it should *not* be considered production-ready code.

  Written in 2019 by Luis Caballero Martínez.
  Released under a Creative Commons CC0 Public Domain Dedication.

  This software is published from: Spain.
  To the extent possible under law, the author have dedicated all
  copyright and related and neighboring rights to this software
  to the public domain worldwide. This software is distributed
  without any warranty.

  You can find a copy of the CC0 Public Domain Dedication at
  < http://creativecommons.org/publicdomain/zero/1.0/ >
}
unit fmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics,
  Dialogs, StdCtrls;

type

  { TMainForm }

  TMainForm = class(TForm)
    SrcList: TListBox;
    lbInfo: TLabel;
    DestList: TListBox;
    procedure FormResize(Sender: TObject);
    procedure SrcListDblClick(Sender: TObject);
    procedure SrcListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SrcListShowHint(Sender: TObject; HintInfo: PHintInfo);

  private
     procedure Decompose(Index: Integer);
  end;

var
  MainForm: TMainForm;

implementation

uses strutils, LCLType;

{$R *.lfm}

resourcestring
  sSrcDefHint   = 'Double-click the item to decompose';
  sClickAString = 'Please, double click in a string';
  sfWithFloat = '%d = %s - As float: %f';
  sfNoFloat   = '%d = %s';

{ TMainForm }

procedure TMainForm.FormResize(Sender: TObject);
{ Make the SrcList occupy half the form }
begin
  SrcList.Width := (ClientWidth div 2) - (SrcList.BorderSpacing.Around * 2);
end;

procedure TMainForm.SrcListDblClick(Sender: TObject);
begin
  if SrcList.ItemIndex >= 0 then
    Decompose(SrcList.ItemIndex)
  else
    ShowMessage(sClickAString);
end;

procedure TMainForm.SrcListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  { Whaping <Enter> does as a double-click }
  if (Key = VK_RETURN) and (Shift = []) then
    SrcListDblClick(Sender);
end;

procedure TMainForm.SrcListShowHint(Sender: TObject; HintInfo: PHintInfo);
{ Adapt the SrcList Hint to where the mouse pointer is }
var
  Index: Integer;
begin
  Index := SrcList.ItemAtPos(HintInfo^.CursorPos, True);
  if Index < 0 then
    HintInfo^.HintStr := sSrcDefHint
  else
    HintInfo^.HintStr := SrcList.Items[Index];
end;

procedure TMainForm.Decompose(Index: Integer);
{ Decompose "Index"th string from SrcList and add the words to DestList }
var
  Count: Integer;
  AWord: String;
  Delims: TSysCharSet;
  AFloat: Extended;
begin
  if Index in [0..SrcList.Count-1] then begin { Sanity check }
    { Select the proper delimiters }
    if SrcList.Items[Index].StartsWith('|') then
      Delims := ['|']     { String format: "|something||other||...| }
    else if SrcList.Items[Index].StartsWith('[') then
      Delims := ['[',']'] { String format: "[something][other][...] }
    else                { Free form string? Use standard delimiters }
      Delims := StdWordDelims;
    { Extract all words }
    DestList.Clear;
    Count := 0;
    repeat
      Inc(Count);
      AWord := ExtractWord(Count, SrcList.Items[Index], Delims);
      if not AWord.IsEmpty then
        if TryStrToFloat(AWord, AFloat) then
          { If a float, show also the StrToFloat conversion}
          DestList.Items.Add(sfWithFloat,[Count, AWord, AFloat])
        else
          DestList.Items.Add(sfNoFloat,[Count, AWord]);
    until AWord.IsEmpty;
    ShowMessageFmt('Found %d words', [Count-1])
  end;
end;

end.

